/*

   Copyright (C) 2013 Andrej Lojdl

   This file is part of Octave.

   Octave is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3 of the License, or (at your
   option) any later version.

   Octave is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with Octave; see the file COPYING.  If not, see
   <http://www.gnu.org/licenses/>.

 */

#if ! defined (txt_latex_h)
#define txt_latex_h 1

#ifdef ENABLE_LATEX

#include <string>
#include <dMatrix.h>
#include <uint8NDArray.h>
#include "txt-render.h"

class 
OCTINTERP_API
latex_render : public base_text_render
{  
public:
  latex_render (void);

  ~latex_render (void);  

  void set_font (const std::string& name, const std::string& weight,
                 const std::string& angle, double size);

  void set_color (Matrix c);

  void text_to_pixels (const std::string& txt,
                       uint8NDArray& pixels_, Matrix& bbox,
                       int halign, int valign, double rotation);

  Matrix get_extent (text_element *elt, double rotation = 0.0);
  Matrix get_extent (const std::string& txt, double rotation = 0.0);
  
  /* method that cheks if there are all required programs installed */
  int check_programs (void);

  /* method making TEX file from text (input string) */ 
  void adapter (const std::string& txt);

  /* method rendering text TEX -> DVI -> EPS -> PNG -> RAW */ 
  uint8NDArray render (void);

private:
  int font_size;
  std::string font_name,directory_path;
  Matrix bbox;
  uint8NDArray pixels;
  uint8NDArray color;

};
#endif
#endif //ENABLE_LATEX
